@@ -1,7 +1,7 @@
 /*
  * jabglue.c    -- Jabber protocol handling
  *
- * Copyright (C) 2005-2007 Mikael Berthe <mikael@lilotux.net>
+ * Copyright (C) 2005-2008 Mikael Berthe <mikael@lilotux.net>
  * Parts come from the centericq project:
  * Copyright (C) 2002-2005 by Konstantin Klyagin <konst@konst.org.ua>
  *
@@ -34,6 +34,7 @@
 #include "commands.h"
 #include "pgp.h"
 #include "otr.h"
+#include "fifo.h"
 
 #define JABBERPORT      5222
 #define JABBERSSLPORT   5223
@@ -109,7 +110,7 @@ char *compose_jid(const char *username, const char *servername,
   return fjid;
 }
 
-inline unsigned char jb_getonline(void)
+unsigned char jb_getonline(void)
 {
   return online;
 }
@@ -231,26 +232,44 @@ void jb_main()
   long tmout;
   struct timeval tv;
   static time_t last_eviqs_check = 0L;
-
-  if (!online) {
-    safe_usleep(10000);
-    check_connection();
-    return;
-  }
-
-  if (jc && jc->state == JCONN_STATE_CONNECTING) {
-    safe_usleep(75000);
-    jab_start(jc);
-    return;
-  }
+  int maxfd = 0;
+  int fifofd;
 
   FD_ZERO(&fds);
   FD_SET(0, &fds);
-  FD_SET(jc->fd, &fds);
+  if (jc && jc->fd > 0) {
+    FD_SET(jc->fd, &fds);
+    maxfd = jc->fd;
+  }
+
+  fifofd = fifo_get_fd();
+  if (fifofd > 0) {
+    FD_SET(fifofd, &fds);
+    maxfd = MAX(maxfd, fifofd);
+  }
 
   tv.tv_sec = 60;
   tv.tv_usec = 0;
 
+  if (!online || (jc && jc->state == JCONN_STATE_CONNECTING)) {
+    if (online) {
+      // We're connecting and we need to reduce the timeout.
+      tv.tv_sec = 0;
+      tv.tv_usec = 250000;
+    } else {
+      tv.tv_sec = 30;
+      // Let's first update the screen, we could sleep for a long time...
+      scr_DoUpdate();
+    }
+    // If we're not connected, sleep for a while...
+    select(maxfd + 1, &fds, NULL, NULL, &tv);
+    if (!online)
+      check_connection();
+    else
+      jab_start(jc);
+    return;
+  }
+
   time(&now);
 
   if (KeepaliveDelay) {
@@ -276,7 +295,7 @@ void jb_main()
     tv.tv_usec = 350000;
 
   scr_DoUpdate();
-  if (select(jc->fd + 1, &fds, NULL, NULL, &tv) > 0) {
+  if (select(maxfd + 1, &fds, NULL, NULL, &tv) > 0) {
     if (FD_ISSET(jc->fd, &fds))
       jab_poll(jc, 0);
   }
@@ -428,7 +447,11 @@ static xmlnode presnew(enum imstatus st, const char *recipient,
         break;
   }
 
-  prio = settings_opt_get_int("priority");
+  if (st == away || st == notavail)
+    prio = settings_opt_get_int("priority_away");
+  else
+    prio = settings_opt_get_int("priority");
+
   if (prio) {
     char strprio[8];
     snprintf(strprio, 8, "%d", (int)prio);
@@ -580,7 +603,11 @@ void jb_send_msg(const char *fjid, const char *text, int type,
   if (encrypted)
     *encrypted = 0;
 
-  if (!online) return;
+  if (!online)
+    return;
+
+  if (!text && type == ROSTER_TYPE_USER)
+    return;
 
   if (type_overwrite)
     strtype = type_overwrite;
@@ -876,7 +903,7 @@ void jb_send_chatstate(gpointer buddy, guint chatstate)
   const char *bjid;
 #ifdef JEP0085
   GSList *resources, *p_res, *p_next;
-  struct jep0085 *jep85 = NULL;;
+  struct jep0085 *jep85 = NULL;
 #endif
 #ifdef JEP0022
   struct jep0022 *jep22;
@@ -2345,7 +2372,7 @@ static void handle_presence_muc(const char *from, xmlnode xmldata,
   if (new_member && autowhois == autowhois_on) {
     // FIXME: This will fail for some UTF-8 nicknames.
     gchar *joiner_nick = from_utf8(rname);
-    room_whois(room_elt->data, joiner_nick, FALSE);
+    cmd_room_whois(room_elt->data, joiner_nick, FALSE);
     g_free(joiner_nick);
   }
 
@@ -2366,6 +2393,13 @@ static void handle_packet_presence(jconn conn, char *type, char *from,
   rname = strchr(from, JID_RESOURCE_SEPARATOR);
   if (rname) rname++;
 
+  if (settings_opt_get_int("ignore_self_presence")) {
+    const char *self_fjid = jid_full(jc->user);
+    if (self_fjid && !strcasecmp(self_fjid, from)) {
+      return; // Ignoring self presence
+    }
+  }
+
   r = jidtodisp(from);
 
   // Check for MUC presence packet
